Package: xtfifevd
Type: Package
Title: Panel Fixed Effects Filtered and Variance Decomposition
        Estimation
Version: 1.0.2
Authors@R: c(
    person("Muhammad", "Alkhalaf", 
           email = "muhammedalkhalaf@gmail.com",
           role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0009-0002-2677-9246")))
Description: Implements fixed effects estimators for time-invariant variables
    in panel data models. Provides three estimation methods: FEVD (Fixed Effects
    Vector Decomposition) from Plumper and Troeger (2007) 
    <doi:10.1093/pan/mpm002>, and FEF (Fixed Effects Filtered) and FEF-IV
    (instrumental variables variant) from Pesaran and Zhou (2018)
    <doi:10.1080/07474938.2016.1222225>. All methods use the correct
    Pesaran-Zhou variance estimators that account for generated regressor
    uncertainty, avoiding the size distortions documented in the literature.
License: GPL-3
URL: https://github.com/muhammedalkhalaf/xtfifevd
BugReports: https://github.com/muhammedalkhalaf/xtfifevd/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: stats, MASS
Suggests: plm, lmtest, sandwich, testthat (>= 3.0.0), knitr, rmarkdown
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-27 19:31:03 UTC; acad_
Author: Muhammad Alkhalaf [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0002-2677-9246>)
Maintainer: Muhammad Alkhalaf <muhammedalkhalaf@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 20:30:02 UTC
Built: R 4.6.0; ; 2026-05-01 20:33:05 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(coef,xtfifevd)
S3method(confint,xtfifevd)
S3method(print,summary.xtfifevd)
S3method(print,xtfifevd)
S3method(summary,xtfifevd)
S3method(vcov,xtfifevd)
export(bw_ratio)
export(fef)
export(fef_iv)
export(fevd)
export(xtfifevd)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,lm.fit)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,vcov)

# xtfifevd

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/xtfifevd)](https://CRAN.R-project.org/package=xtfifevd)
<!-- badges: end -->

## Overview

**xtfifevd** implements fixed effects estimators for time-invariant variables
in panel data models. Standard fixed effects (FE) estimation cannot identify
coefficients on time-invariant regressors because they are collinear with the
individual fixed effects. This package provides three methods to estimate these
coefficients:

- **FEVD**: Fixed Effects Vector Decomposition (Plümper & Troeger, 2007)
- **FEF**: Fixed Effects Filtered (Pesaran & Zhou, 2018)
- **FEF-IV**: FEF with instrumental variables for endogenous regressors

All methods use the correct **Pesaran-Zhou (2018) variance estimators** that
account for generated regressor uncertainty, avoiding the severe size
distortions documented in the literature.

## Installation

```r
# Install from CRAN (when available)
install.packages("xtfifevd")

# Or install development version from GitHub
# install.packages("remotes")
```

## Usage

```r
library(xtfifevd)

# Simulate panel data
set.seed(123)
N <- 100  # panels
T <- 10   # time periods
n <- N * T

id <- rep(1:N, each = T)
time <- rep(1:T, N)
alpha_i <- rep(rnorm(N), each = T)  # Fixed effects
z <- rep(rnorm(N), each = T)        # Time-invariant variable
x <- rnorm(n)                        # Time-varying variable
y <- 1 + 2 * x + 0.5 * z + alpha_i + rnorm(n, sd = 0.5)

data <- data.frame(id = id, time = time, y = y, x = x, z = z)

# Formula: y ~ time_varying_vars | time_invariant_vars
fit <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time")
summary(fit)
```

Output:
```
======================================================================
FEVD Estimation Results                              xtfifevd 1.0.0
======================================================================
Dep. variable:   y
Method:          FEVD
Variance:        Pesaran-Zhou (2016)
Observations:    1000       Groups:     100
T (average):     10.00
----------------------------------------------------------------------

            Estimate Std. Error z value Pr(>|z|)    
x            1.99842    0.01621 123.283  < 2e-16 ***
z            0.51247    0.10834   4.731 2.23e-06 ***
_cons        0.98123    0.10521   9.326  < 2e-16 ***
---
Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

----------------------------------------------------------------------
Time-varying (FE):      x
Time-invariant:         z
sigma_e: 0.5012    sigma_u: 0.9834
======================================================================
```

## Methods Comparison

```r
# All three methods
fit_fevd <- fevd(y ~ x | z, data, id = "id", time = "time")
fit_fef  <- fef(y ~ x | z, data, id = "id", time = "time")

# FEF and FEVD produce identical point estimates (Proposition 3)
all.equal(coef(fit_fevd), coef(fit_fef))
# [1] TRUE

# With instruments (when z may be endogenous)
data$iv <- data$z + rnorm(n, sd = 0.3)  # Instrument
fit_iv <- fef_iv(y ~ x | z, data, id = "id", time = "time",
                 instruments = ~ iv)
```

## Diagnostics

```r
# Between/Within variance ratio
# High ratios suggest FEVD/FEF may improve on standard FE
bw_ratio(data, c("z", "x"), id = "id")

# Between/Within Variance Ratios
# ------------------------------------------------------------
# Variable        Between SD    Within SD     B/W Ratio
# ------------------------------------------------------------
# z                   1.0234       0.0000          Inf
# x                   0.1023       0.9876         0.10
# ------------------------------------------------------------
# Note: B/W > 1.7 suggests FEVD/FEF may improve on FE
```

## Why Use This Package?

### The Problem

Standard FE estimation "absorbs" time-invariant variables into the fixed
effects, making their coefficients unidentified. Researchers often want to
estimate effects of variables like:

- Gender, ethnicity, geographic region
- Institutional characteristics
- Baseline/entry values

### Common (Wrong) Solutions

1. **Hausman-Taylor**: Requires valid instruments, often hard to justify
2. **Naive FEVD Stage 3 SEs**: Severely understated (factors of 2-5x!)
3. **Ignoring the problem**: Biased pooled OLS

### The Right Solution

FEVD/FEF methods with **Pesaran-Zhou corrected standard errors** provide:

- Consistent point estimates under standard FE assumptions
- Valid inference that accounts for generated regressor uncertainty
- Better efficiency than FE for variables with high between/within ratio

## References

- Plümper, T., & Troeger, V. E. (2007). Efficient Estimation of Time-Invariant
  and Rarely Changing Variables in Finite Sample Panel Analyses with Unit Fixed
  Effects. *Political Analysis*, 15(2), 124-139.
  [doi:10.1093/pan/mpm002](https://doi.org/10.1093/pan/mpm002)

- Pesaran, M. H., & Zhou, Q. (2018). Estimation of time-invariant effects in
  static panel data models. *Econometric Reviews*, 37(10), 1137-1171.
  [doi:10.1080/07474938.2016.1222225](https://doi.org/10.1080/07474938.2016.1222225)

- Greene, W. H. (2011). Fixed Effects Vector Decomposition: A Magical Solution
  to the Problem of Time-Invariant Variables in Fixed Effects Models?
  *Political Analysis*, 19(2), 135-146.
  [doi:10.1093/pan/mpq034](https://doi.org/10.1093/pan/mpq034)

## License

GPL-3

#' Between/Within Variance Ratio for Time-Invariant Variables
#'
#' @importFrom stats sd
#' @description
#' Computes the between-panel and within-panel standard deviations for
#' specified variables, along with their ratio. This diagnostic helps assess
#' whether FEVD/FEF methods may improve upon standard FE estimation.
#'
#' @param data A data frame containing the panel data.
#' @param variables A character vector of variable names to analyze.
#' @param id Character string naming the panel identifier variable.
#'
#' @return A data frame with columns:
#' \describe{
#'   \item{variable}{Variable name}
#'   \item{sd_between}{Between-panel standard deviation}
#'   \item{sd_within}{Within-panel standard deviation}
#'   \item{bw_ratio}{Ratio of between to within SD}
#' }
#'
#' @details
#' For truly time-invariant variables, the within-panel SD should be zero
#' (or near-zero due to numerical precision), giving an infinite ratio.
#'
#' According to Plümper and Troeger (2007), FEVD/FEF methods tend to improve
#' upon standard FE when:
#' \itemize{
#'   \item The between/within ratio exceeds approximately 1.7
#'   \item The correlation between z and the unobserved unit effect is not too high
#' }
#'
#' @examples
#' # Create example data
#' set.seed(42)
#' N <- 50
#' T <- 5
#' id <- rep(1:N, each = T)
#' z_invariant <- rep(rnorm(N), each = T)  # Truly time-invariant
#' z_slow <- rep(rnorm(N), each = T) + rnorm(N * T, sd = 0.1)  # Slowly varying
#' x_varying <- rnorm(N * T)  # Time-varying
#'
#' data <- data.frame(id = id, z_inv = z_invariant,
#'                    z_slow = z_slow, x = x_varying)
#'
#' bw_ratio(data, c("z_inv", "z_slow", "x"), id = "id")
#'
#' @references
#' Plumper, T., & Troeger, V. E. (2007). Efficient Estimation of Time-Invariant
#' and Rarely Changing Variables in Finite Sample Panel Analyses with Unit Fixed
#' Effects. \emph{Political Analysis}, 15(2), 124-139.
#' \doi{10.1093/pan/mpm002}
#'
#' @export
bw_ratio <- function(data, variables, id) {
  
  # Validate inputs
  if (!is.data.frame(data)) {
    stop("'data' must be a data frame")
  }
  
  missing_vars <- setdiff(c(variables, id), names(data))
  if (length(missing_vars) > 0) {
    stop("Variables not found in data: ", paste(missing_vars, collapse = ", "))
  }
  
  panel_id <- data[[id]]
  
  results <- data.frame(
    variable = character(),
    sd_between = numeric(),
    sd_within = numeric(),
    bw_ratio = numeric(),
    stringsAsFactors = FALSE
  )
  
  for (v in variables) {
    x <- data[[v]]
    
    # Between SD: SD of panel means
    panel_means <- tapply(x, panel_id, mean, na.rm = TRUE)
    sd_between <- sd(panel_means, na.rm = TRUE)
    
    # Within SD: SD of deviations from panel means
    mean_expanded <- panel_means[as.character(panel_id)]
    deviations <- x - mean_expanded
    sd_within <- sd(deviations, na.rm = TRUE)
    
    # Ratio
    if (sd_within > 1e-10) {
      ratio <- sd_between / sd_within
    } else {
      ratio <- Inf
    }
    
    results <- rbind(results, data.frame(
      variable = v,
      sd_between = sd_between,
      sd_within = sd_within,
      bw_ratio = ratio,
      stringsAsFactors = FALSE
    ))
  }
  
  # Print nicely via message() so output can be suppressed
  lines <- c(
    "",
    "Between/Within Variance Ratios",
    paste(rep("-", 60), collapse = ""),
    sprintf("%-15s %12s %12s %12s",
            "Variable", "Between SD", "Within SD", "B/W Ratio"),
    paste(rep("-", 60), collapse = "")
  )
  for (i in seq_len(nrow(results))) {
    if (is.finite(results$bw_ratio[i])) {
      lines <- c(lines, sprintf("%-15s %12.4f %12.4f %12.2f",
                  results$variable[i],
                  results$sd_between[i],
                  results$sd_within[i],
                  results$bw_ratio[i]))
    } else {
      lines <- c(lines, sprintf("%-15s %12.4f %12.4f %12s",
                  results$variable[i],
                  results$sd_between[i],
                  results$sd_within[i],
                  "Inf"))
    }
  }
  lines <- c(
    lines,
    paste(rep("-", 60), collapse = ""),
    "Note: B/W > 1.7 suggests FEVD/FEF may improve on FE",
    "      (threshold depends on corr(z,u); see Plumper & Troeger 2007)",
    ""
  )
  message(paste(lines, collapse = "\n"))
  
  invisible(results)
}

#' Methods for xtfifevd objects
#'
#' @name xtfifevd-methods
#' @keywords internal
#' @importFrom stats coef vcov pnorm qnorm printCoefmat
NULL


#' @export
print.xtfifevd <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
  
  cat("\n")
  cat(x$method, "Estimation Results\n")
  cat(rep("-", 50), sep = "")
  cat("\n\n")
  
  cat("Dep. variable:", x$y_name, "\n")
  cat("Method:       ", x$method, "\n")
  cat("Observations: ", x$N, "\n")
  cat("Groups:       ", x$N_g, "\n")
  cat("T (average):  ", round(x$T_bar, 2), "\n")
  cat("\n")
  
  cat("Coefficients:\n")
  print(round(x$coefficients, digits))
  cat("\n")
  
  invisible(x)
}


#' @export
summary.xtfifevd <- function(object, ...) {
  
  coefs <- object$coefficients
  se <- sqrt(diag(object$vcov))
  z_val <- coefs / se
  p_val <- 2 * pnorm(-abs(z_val))
  
  coef_table <- cbind(
    Estimate = coefs,
    `Std. Error` = se,
    `z value` = z_val,
    `Pr(>|z|)` = p_val
  )
  
  # Add significance stars
  stars <- ifelse(p_val < 0.001, "***",
                  ifelse(p_val < 0.01, "**",
                         ifelse(p_val < 0.05, "*",
                                ifelse(p_val < 0.1, ".", ""))))
  
  result <- list(
    call = object$call,
    method = object$method,
    coefficients = coef_table,
    stars = stars,
    N = object$N,
    N_g = object$N_g,
    T_bar = object$T_bar,
    sigma2_e = object$sigma2_e,
    sigma2_u = object$sigma2_u,
    y_name = object$y_name,
    x_names = object$x_names,
    z_names = object$z_names,
    k_x = object$k_x,
    k_z = object$k_z
  )
  
  class(result) <- "summary.xtfifevd"
  result
}


#' @export
print.summary.xtfifevd <- function(x, digits = max(3L, getOption("digits") - 3L),
                                   signif.stars = getOption("show.signif.stars"),
                                   ...) {
  
  cat("\n")
  cat(rep("=", 70), sep = "")
  cat("\n")
  cat(x$method, "Estimation Results", "                      xtfifevd 1.0.0\n")
  cat(rep("=", 70), sep = "")
  cat("\n")
  
  cat("Dep. variable:  ", x$y_name, "\n")
  cat("Method:         ", x$method, "\n")
  cat("Variance:        Pesaran-Zhou (2016)\n")
  cat(sprintf("Observations:    %-10d Groups:     %d\n", x$N, x$N_g))
  cat(sprintf("T (average):     %-10.2f\n", x$T_bar))
  cat(rep("-", 70), sep = "")
  cat("\n\n")
  
  # Print coefficients
  printCoefmat(x$coefficients, digits = digits, signif.stars = signif.stars,
               P.values = TRUE, has.Pvalue = TRUE, ...)
  
  cat("\n")
  cat(rep("-", 70), sep = "")
  cat("\n")
  cat("Time-varying (FE):     ", paste(x$x_names, collapse = ", "), "\n")
  cat("Time-invariant:        ", paste(x$z_names, collapse = ", "), "\n")
  cat(sprintf("sigma_e: %.4f    sigma_u: %.4f\n",
              sqrt(x$sigma2_e), sqrt(x$sigma2_u)))
  cat(rep("=", 70), sep = "")
  cat("\n")
  
  invisible(x)
}


#' @export
coef.xtfifevd <- function(object, ...) {
  object$coefficients
}


#' @export
vcov.xtfifevd <- function(object, ...) {
  object$vcov
}


#' @export
confint.xtfifevd <- function(object, parm, level = 0.95, ...) {
  
  cf <- coef(object)
  se <- sqrt(diag(vcov(object)))
  
  if (missing(parm)) {
    parm <- names(cf)
  } else if (is.numeric(parm)) {
    parm <- names(cf)[parm]
  }
  
  a <- (1 - level) / 2
  fac <- qnorm(1 - a)
  
  ci <- cbind(cf[parm] - fac * se[parm],
              cf[parm] + fac * se[parm])
  
  pct <- paste0(format(100 * c(a, 1 - a), digits = 3), "%")
  colnames(ci) <- pct
  rownames(ci) <- parm
  
  ci
}

#' Panel Fixed Effects Estimation for Time-Invariant Variables
#'
#' @description
#' Estimates panel models with time-invariant regressors using FEVD, FEF, or
#' FEF-IV methods. Standard fixed effects estimation cannot identify
#' coefficients on time-invariant variables; these methods decompose or filter
#' the unit effects to recover these coefficients.
#'
#' @param formula A formula of the form `y ~ x1 + x2 | z1 + z2` where variables
#'   before `|` are time-varying and variables after `|` are time-invariant.
#' @param data A data frame containing the variables.
#' @param id Character string naming the panel (individual) identifier variable.
#' @param time Character string naming the time identifier variable.
#' @param method Estimation method: `"fevd"` (default), `"fef"`, or `"fef_iv"`.
#' @param instruments For `method = "fef_iv"`, a one-sided formula specifying

#'   instrumental variables, e.g., `~ iv1 + iv2`.
#' @param na.action How to handle missing values. Default is `na.omit`.
#'
#' @return An object of class `"xtfifevd"` containing:
#' \describe{
#'   \item{coefficients}{Named vector of all coefficients (beta, gamma, intercept)}
#'   \item{vcov}{Variance-covariance matrix using Pesaran-Zhou estimator}
#'   \item{beta}{Coefficients on time-varying variables (from FE stage)}
#'   \item{gamma}{Coefficients on time-invariant variables}
#'   \item{intercept}{Overall intercept}
#'   \item{residuals}{Idiosyncratic residuals from FE stage}
#'   \item{fitted.values}{Fitted values}
#'   \item{sigma2_e}{Variance of idiosyncratic error}
#'   \item{sigma2_u}{Variance of unit effects}
#'   \item{N}{Total number of observations}
#'   \item{N_g}{Number of groups (panels)}
#'   \item{T_bar}{Average time periods per panel}
#'   \item{method}{Estimation method used}
#'   \item{call}{The matched call}
#' }
#'
#' @details
#' ## Model
#' The panel model is:
#' \deqn{y_{it} = \alpha_i + x_{it}'\beta + z_i'\gamma + \varepsilon_{it}}
#'
#' where \eqn{x_{it}} are time-varying regressors, \eqn{z_i} are time-invariant
#' regressors, and \eqn{\alpha_i} are individual fixed effects.
#'
#' ## Stage 1 (All Methods)
#' Fixed effects regression of \eqn{y_{it}} on \eqn{x_{it}} yields consistent
#' \eqn{\hat{\beta}} and combined residuals \eqn{\hat{u}_{it} = \hat{\alpha}_i + \hat{\varepsilon}_{it}}.
#'
#' ## Stage 2
#' Time-averaged residuals \eqn{\bar{u}_i} are regressed on \eqn{z_i}:
#' \itemize{
#'   \item **FEF**: OLS of \eqn{\bar{u}_i} on \eqn{z_i} with intercept
#'   \item **FEF-IV**: 2SLS using instruments \eqn{r_i}
#'   \item **FEVD**: Same as FEF, then Stage 3 pooled OLS (point estimates identical)
#' }
#'
#' ## Variance Estimation
#' Uses Pesaran-Zhou (2016) Equation 17/51 which properly accounts for
#' estimation uncertainty from Stage 1. The naive pooled OLS standard errors
#' from FEVD Stage 3 are **inconsistent** and can understate true SEs by
#' factors of 2-5x or more.
#'
#' @examples
#' # Simulate panel data
#' set.seed(123)
#' N <- 100  # panels
#' T <- 10   # time periods
#' n <- N * T
#'
#' # Generate data
#' id <- rep(1:N, each = T)
#' time <- rep(1:T, N)
#' alpha_i <- rep(rnorm(N), each = T)  # Fixed effects
#' z <- rep(rnorm(N), each = T)        # Time-invariant
#' x <- rnorm(n)                        # Time-varying
#' y <- 1 + 2 * x + 0.5 * z + alpha_i + rnorm(n, sd = 0.5)
#'
#' data <- data.frame(id = id, time = time, y = y, x = x, z = z)
#'
#' # Estimate with different methods
#' fit_fevd <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time")
#' summary(fit_fevd)
#'
#' fit_fef <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time",
#'                     method = "fef")
#' summary(fit_fef)
#'
#' @references
#' Plumper, T., & Troeger, V. E. (2007). Efficient Estimation of Time-Invariant
#' and Rarely Changing Variables in Finite Sample Panel Analyses with Unit Fixed
#' Effects. \emph{Political Analysis}, 15(2), 124-139.
#' \doi{10.1093/pan/mpm002}
#'
#' Pesaran, M. H., & Zhou, Q. (2018). Estimation of time-invariant effects in
#' static panel data models. \emph{Econometric Reviews}, 37(10), 1137-1171.
#' \doi{10.1080/07474938.2016.1222225}
#'
#' @seealso [fevd()], [fef()], [fef_iv()], [bw_ratio()]
#'
#' @export
xtfifevd <- function(formula, data, id, time,
                     method = c("fevd", "fef", "fef_iv"),
                     instruments = NULL,
                     na.action = na.omit) {
  
  call <- match.call()
  method <- match.arg(method)
  
  # Parse the formula: y ~ x1 + x2 | z1 + z2
  parsed <- .parse_formula(formula, data, id, time, instruments, na.action)
  
  # Dispatch to appropriate estimator
  result <- switch(method,
                   "fevd" = .estimate_fevd(parsed),
                   "fef" = .estimate_fef(parsed),
                   "fef_iv" = .estimate_fef_iv(parsed))
  
  result$call <- call
  result$formula <- formula
  result$method <- toupper(method)
  result$method <- gsub("_", "-", result$method)
  
  class(result) <- "xtfifevd"
  result
}


#' @describeIn xtfifevd FEVD estimation (3-stage, Plümper-Troeger)
#' @export
fevd <- function(formula, data, id, time, na.action = na.omit) {
  xtfifevd(formula, data, id, time, method = "fevd", na.action = na.action)
}


#' @describeIn xtfifevd FEF estimation (2-stage, Pesaran-Zhou)
#' @export
fef <- function(formula, data, id, time, na.action = na.omit) {
  xtfifevd(formula, data, id, time, method = "fef", na.action = na.action)
}


#' @describeIn xtfifevd FEF-IV estimation with instruments
#' @export
fef_iv <- function(formula, data, id, time, instruments,
                   na.action = na.omit) {
  xtfifevd(formula, data, id, time, method = "fef_iv",
           instruments = instruments, na.action = na.action)
}

# Tests for xtfifevd package

test_that("xtfifevd estimates time-varying coefficients correctly", {
  # Simulate data with known coefficients
  set.seed(123)
  N <- 100
  T <- 10
  n <- N * T
  
  id <- rep(1:N, each = T)
  time <- rep(1:T, N)
  alpha_i <- rep(rnorm(N, sd = 1), each = T)
  z <- rep(rnorm(N), each = T)
  x <- rnorm(n)
  
  # True model: y = 1 + 2*x + 0.5*z + alpha_i + eps
  y <- 1 + 2 * x + 0.5 * z + alpha_i + rnorm(n, sd = 0.5)
  
  data <- data.frame(id = id, time = time, y = y, x = x, z = z)
  
  # FEVD estimation
  fit <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time")
  
  # Check that time-varying coefficient is close to true value

  expect_equal(fit$beta["x"], c(x = 2), tolerance = 0.2)
  
  # Check that time-invariant coefficient is estimated
  expect_equal(fit$gamma["z"], c(z = 0.5), tolerance = 0.3)
  
  # Check structure

  expect_s3_class(fit, "xtfifevd")
  expect_equal(fit$method, "FEVD")
  expect_equal(fit$N, n)
  expect_equal(fit$N_g, N)
})


test_that("FEF produces same point estimates as FEVD", {
  set.seed(456)
  N <- 50
  T <- 8
  n <- N * T
  
  id <- rep(1:N, each = T)
  time <- rep(1:T, N)
  alpha_i <- rep(rnorm(N), each = T)
  z <- rep(runif(N, -1, 1), each = T)
  x <- rnorm(n)
  
  y <- 0.5 + 1.5 * x + 0.8 * z + alpha_i + rnorm(n, sd = 0.3)
  data <- data.frame(id = id, time = time, y = y, x = x, z = z)
  
  fit_fevd <- fevd(y ~ x | z, data = data, id = "id", time = "time")
  fit_fef <- fef(y ~ x | z, data = data, id = "id", time = "time")
  
  # Point estimates should be identical (Pesaran-Zhou Proposition 3)
  expect_equal(fit_fevd$beta, fit_fef$beta, tolerance = 1e-10)
  expect_equal(fit_fevd$gamma, fit_fef$gamma, tolerance = 1e-10)
})


test_that("FEF-IV requires instruments", {
  data <- data.frame(
    id = rep(1:10, each = 5),
    time = rep(1:5, 10),
    y = rnorm(50),
    x = rnorm(50),
    z = rep(rnorm(10), each = 5)
  )
  
  expect_error(
    fef_iv(y ~ x | z, data = data, id = "id", time = "time"),
    "instruments"
  )
})


test_that("FEF-IV works with valid instruments", {
  set.seed(789)
  N <- 60
  T <- 6
  n <- N * T
  
  id <- rep(1:N, each = T)
  time <- rep(1:T, N)
  
  # Generate instrument correlated with z but not with error
  iv <- rep(rnorm(N), each = T)
  z <- iv + rep(rnorm(N, sd = 0.5), each = T)  # z correlated with iv
  z <- rep(tapply(z, id, mean), each = T)      # Make time-invariant
  
  alpha_i <- rep(rnorm(N), each = T)
  x <- rnorm(n)
  y <- 1 + x + 0.5 * z + alpha_i + rnorm(n, sd = 0.4)
  
  data <- data.frame(id = id, time = time, y = y, x = x, z = z, iv = iv)
  
  fit <- fef_iv(y ~ x | z, data = data, id = "id", time = "time",
                instruments = ~ iv)
  
  expect_s3_class(fit, "xtfifevd")
  expect_equal(fit$method, "FEF-IV")
  expect_true(!is.null(fit$gamma))
})


test_that("summary.xtfifevd produces correct output", {
  set.seed(101)
  N <- 40
  T <- 5
  
  data <- data.frame(
    id = rep(1:N, each = T),
    time = rep(1:T, N),
    y = rnorm(N * T),
    x = rnorm(N * T),
    z = rep(rnorm(N), each = T)
  )
  
  fit <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time")
  s <- summary(fit)
  
  expect_s3_class(s, "summary.xtfifevd")
  expect_true("coefficients" %in% names(s))
  expect_equal(nrow(s$coefficients), 3)  # x, z, _cons
})


test_that("vcov returns correct structure", {
  set.seed(202)
  N <- 30
  T <- 4
  
  data <- data.frame(
    id = rep(1:N, each = T),
    time = rep(1:T, N),
    y = rnorm(N * T),
    x1 = rnorm(N * T),
    x2 = rnorm(N * T),
    z = rep(rnorm(N), each = T)
  )
  
  fit <- xtfifevd(y ~ x1 + x2 | z, data = data, id = "id", time = "time")
  V <- vcov(fit)
  
  expect_true(is.matrix(V))
  expect_equal(nrow(V), ncol(V))
  expect_equal(nrow(V), 4)  # x1, x2, z, _cons
  
  # Check symmetry
  expect_equal(V, t(V))
  
  # Check positive definiteness (all eigenvalues positive)
  expect_true(all(eigen(V, symmetric = TRUE, only.values = TRUE)$values > 0))
})


test_that("bw_ratio computes correctly", {
  set.seed(303)
  N <- 20
  T <- 6
  
  data <- data.frame(
    id = rep(1:N, each = T),
    z_const = rep(rnorm(N), each = T),   # Time-invariant
    z_vary = rnorm(N * T)                 # Time-varying
  )
  
  result <- bw_ratio(data, c("z_const", "z_vary"), id = "id")
  
  # Time-invariant variable should have within SD near 0
  expect_lt(result$sd_within[result$variable == "z_const"], 1e-10)
  
  # Time-varying variable should have finite ratio
  expect_true(is.finite(result$bw_ratio[result$variable == "z_vary"]))
})


test_that("confint produces valid intervals", {
  set.seed(404)
  N <- 50
  T <- 5
  
  data <- data.frame(
    id = rep(1:N, each = T),
    time = rep(1:T, N),
    y = rnorm(N * T),
    x = rnorm(N * T),
    z = rep(rnorm(N), each = T)
  )
  
  fit <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time")
  ci <- confint(fit)
  
  expect_equal(nrow(ci), 3)
  expect_equal(ncol(ci), 2)
  
  # Lower bound should be less than upper bound
  expect_true(all(ci[, 1] < ci[, 2]))
  
  # Check that coefficients are within intervals
  coefs <- coef(fit)
  for (i in seq_along(coefs)) {
    expect_gte(coefs[i], ci[i, 1])
    expect_lte(coefs[i], ci[i, 2])
  }
})


test_that("formula parsing handles multiple variables", {
  set.seed(505)
  N <- 40
  T <- 5
  n <- N * T
  
  data <- data.frame(
    id = rep(1:N, each = T),
    time = rep(1:T, N),
    y = rnorm(n),
    x1 = rnorm(n),
    x2 = rnorm(n),
    x3 = rnorm(n),
    z1 = rep(rnorm(N), each = T),
    z2 = rep(rnorm(N), each = T)
  )
  
  fit <- xtfifevd(y ~ x1 + x2 + x3 | z1 + z2,
                  data = data, id = "id", time = "time")
  
  expect_equal(fit$k_x, 3)
  expect_equal(fit$k_z, 2)
  expect_equal(length(fit$coefficients), 6)  # 3 x + 2 z + 1 cons
})

#' xtfifevd: Panel Fixed Effects Filtered and Variance Decomposition Estimation
#'
#' @description
#' Implements fixed effects estimators for time-invariant variables in panel
#' data models. Standard fixed effects (FE) estimation cannot identify
#' coefficients on time-invariant regressors because they are collinear with

#' the individual fixed effects. This package provides three methods to estimate
#' these coefficients:
#'
#' @section Methods:
#'
#' \describe{
#'   \item{FEVD}{Fixed Effects Vector Decomposition (Plümper & Troeger, 2007).
#'     A three-stage estimator that decomposes the unit effects into explained
#'     (by time-invariant variables) and unexplained components.}
#'   \item{FEF}{Fixed Effects Filtered (Pesaran & Zhou, 2016). A two-stage
#'     estimator that regresses time-averaged FE residuals on time-invariant
#'     variables.}
#'   \item{FEF-IV}{Fixed Effects Filtered with Instrumental Variables
#'     (Pesaran & Zhou, 2016). Uses external instruments when time-invariant
#'     variables are endogenous.}
#' }
#'
#' @section Variance Estimation:
#' All methods use the Pesaran-Zhou (2016) variance estimator (Equation 17 for
#' FEF/FEVD, Equation 51 for FEF-IV). These correct variance estimators account
#' for the generated regressor problem and avoid the severe size distortions
#' documented for naive pooled OLS standard errors.
#'
#' @references
#' Plumper, T., & Troeger, V. E. (2007). Efficient Estimation of Time-Invariant
#' and Rarely Changing Variables in Finite Sample Panel Analyses with Unit Fixed
#' Effects. \emph{Political Analysis}, 15(2), 124-139.
#' \doi{10.1093/pan/mpm002}
#'
#' Pesaran, M. H., & Zhou, Q. (2018). Estimation of time-invariant effects in
#' static panel data models. \emph{Econometric Reviews}, 37(10), 1137-1171.
#' \doi{10.1080/07474938.2016.1222225}
#'
#' Greene, W. H. (2011). Fixed Effects Vector Decomposition: A Magical Solution
#' to the Problem of Time-Invariant Variables in Fixed Effects Models?
#' \emph{Political Analysis}, 19(2), 135-146.
#' \doi{10.1093/pan/mpq034}
#'
#' @docType package
#' @name xtfifevd-package
#' @aliases xtfifevd-package
#' @keywords internal
"_PACKAGE"

#' Internal estimation functions
#'
#' @name estimation-internal
#' @keywords internal
#' @importFrom stats as.formula coef fitted lm.fit na.omit residuals var
NULL


#' Parse the xtfifevd formula
#' @keywords internal
#' @noRd
.parse_formula <- function(formula, data, id, time, instruments = NULL,
                           na.action = na.omit) {
  
  # Extract formula components
  formula_str <- as.character(formula)
  if (length(formula_str) != 3) {
    stop("Formula must be of the form: y ~ x1 + x2 | z1 + z2")
  }
  
  # Parse y ~ x | z structure
  rhs <- formula_str[3]
  lhs <- formula_str[2]
  
  if (!grepl("\\|", rhs)) {
    stop("Formula must include '|' to separate time-varying (x) and ",
         "time-invariant (z) variables.\n",
         "Example: y ~ x1 + x2 | z1 + z2")
  }
  
  parts <- strsplit(rhs, "\\|")[[1]]
  x_part <- trimws(parts[1])
  z_part <- trimws(parts[2])
  
  # Build formulas for extraction
  y_formula <- as.formula(paste("~", lhs))
  x_formula <- as.formula(paste("~", x_part))
  z_formula <- as.formula(paste("~", z_part))
  
  # Get variable names
  y_name <- all.vars(y_formula)
  x_names <- all.vars(x_formula)
  z_names <- all.vars(z_formula)
  iv_names <- NULL
  
  if (!is.null(instruments)) {
    iv_names <- all.vars(instruments)
  }
  
  # Check all variables exist
  all_vars <- c(y_name, x_names, z_names, iv_names, id, time)
  missing_vars <- setdiff(all_vars, names(data))
  if (length(missing_vars) > 0) {
    stop("Variables not found in data: ", paste(missing_vars, collapse = ", "))
  }
  
  # Subset and handle NAs
  vars_needed <- unique(c(y_name, x_names, z_names, iv_names, id, time))
  data_sub <- data[, vars_needed, drop = FALSE]
  data_sub <- na.action(data_sub)
  
  # Sort by id, time
  data_sub <- data_sub[order(data_sub[[id]], data_sub[[time]]), ]
  
  # Extract matrices
  y <- data_sub[[y_name]]
  X <- as.matrix(data_sub[, x_names, drop = FALSE])
  Z <- as.matrix(data_sub[, z_names, drop = FALSE])
  
  panel_id <- data_sub[[id]]
  time_id <- data_sub[[time]]
  
  IV <- NULL
  if (!is.null(iv_names)) {
    IV <- as.matrix(data_sub[, iv_names, drop = FALSE])
  }
  
  # Panel info
  panels <- unique(panel_id)
  N_g <- length(panels)
  N <- length(y)
  
  # Compute panel sizes
  panel_sizes <- table(panel_id)
  T_bar <- mean(panel_sizes)
  T_min <- min(panel_sizes)
  T_max <- max(panel_sizes)
  
  # Check balanced
  balanced <- (T_min == T_max)
  
  # Validate time-invariant variables
  for (z_name in z_names) {
    z_var <- data_sub[[z_name]]
    by_panel <- tapply(z_var, panel_id, function(x) length(unique(x)))
    if (any(by_panel > 1)) {
      warning("Variable '", z_name, "' is not strictly time-invariant. ",
              "Results may be unreliable.")
    }
  }
  
  list(
    y = y,
    X = X,
    Z = Z,
    IV = IV,
    panel_id = panel_id,
    time_id = time_id,
    y_name = y_name,
    x_names = x_names,
    z_names = z_names,
    iv_names = iv_names,
    id_name = id,
    time_name = time,
    panels = panels,
    N = N,
    N_g = N_g,
    T_bar = T_bar,
    T_min = T_min,
    T_max = T_max,
    balanced = balanced,
    data = data_sub
  )
}


#' Fixed Effects estimation (Stage 1)
#' @keywords internal
#' @noRd
.fe_stage1 <- function(y, X, panel_id) {
  
  N <- length(y)
  k_x <- ncol(X)
  panels <- unique(panel_id)
  N_g <- length(panels)
  
  # Compute panel means
  panel_means <- function(v) {
    tapply(v, panel_id, mean)[as.character(panel_id)]
  }
  
  y_bar <- panel_means(y)
  X_bar <- apply(X, 2, panel_means)
  if (!is.matrix(X_bar)) X_bar <- matrix(X_bar, ncol = k_x)
  
  # Within transformation
  y_within <- y - y_bar
  X_within <- X - X_bar
  
  # FE regression (demeaned)
  fit_fe <- lm.fit(X_within, y_within)
  beta <- coef(fit_fe)
  names(beta) <- colnames(X)
  
  # Residuals
  e_it <- residuals(fit_fe)  # idiosyncratic residuals
  u_it <- y - X %*% beta     # combined: alpha_i + e_it
  
  # Time-averaged residual (= alpha_i + mean(e_it))
  u_bar <- tapply(u_it, panel_id, mean)
  
  # Variance components
  sigma2_e <- sum(e_it^2) / (N - N_g - k_x)
  
  # Between variance from panel means
  overall_mean <- mean(y - X %*% beta)
  alpha_hat <- u_bar - overall_mean
  sigma2_u <- var(alpha_hat)
  
  # Robust variance for beta (clustered by panel)
  # Bread: (X'X)^{-1}
  XtX <- crossprod(X_within)
  XtX_inv <- solve(XtX)
  
  # Meat: sum of cluster-specific outer products
  # For clustered (panel-robust) standard errors
  meat <- matrix(0, k_x, k_x)
  for (i in panels) {
    idx <- which(panel_id == i)
    Xi <- X_within[idx, , drop = FALSE]
    ei <- e_it[idx]
    # Compute cluster score: X_i' e_i (k_x x 1 vector)
    score_i <- as.vector(crossprod(Xi, ei))
    meat <- meat + outer(score_i, score_i)
  }
  
  V_beta_robust <- XtX_inv %*% meat %*% XtX_inv
  
  # Standard (non-robust) variance
  V_beta <- sigma2_e * XtX_inv
  
  # Compute X_bar_i as a matrix (N_g x k_x)
  X_bar_i <- t(sapply(panels, function(i) {
    colMeans(X[panel_id == i, , drop = FALSE])
  }))
  if (k_x == 1) {
    X_bar_i <- matrix(X_bar_i, ncol = 1)
    colnames(X_bar_i) <- colnames(X)
  }
  
  list(
    beta = beta,
    e_it = e_it,
    u_it = as.vector(u_it),
    u_bar = u_bar,
    y_bar_i = tapply(y, panel_id, mean),
    X_bar_i = X_bar_i,
    sigma2_e = sigma2_e,
    sigma2_u = sigma2_u,
    V_beta = V_beta,
    V_beta_robust = V_beta_robust,
    df_residual = N - N_g - k_x
  )
}


#' FEF Stage 2 estimation
#' @keywords internal
#' @noRd
.fef_stage2 <- function(u_bar, Z, panel_id, panels) {
  
  N_g <- length(panels)
  k_z <- ncol(Z)
  
  # Get one observation per panel (first row of each panel)
  first_row <- !duplicated(panel_id)
  Z_panel <- Z[first_row, , drop = FALSE]
  
  # OLS of u_bar on Z with intercept
  Z_aug <- cbind(1, Z_panel)
  fit <- lm.fit(Z_aug, u_bar)
  
  coefs <- coef(fit)
  alpha <- coefs[1]
  gamma <- coefs[-1]
  names(gamma) <- colnames(Z)
  
  # Residuals: chat_i = ubar_i - alpha - Z_i' gamma
  chat <- residuals(fit)
  
  list(
    gamma = gamma,
    alpha = alpha,
    chat = chat,
    Z_panel = Z_panel
  )
}


#' Pesaran-Zhou variance for gamma (Eq. 17)
#' @keywords internal
#' @noRd
.pz_variance_fef <- function(Z_panel, X_bar_i, chat, V_beta_robust, N_g) {
  
  k_z <- ncol(Z_panel)
  k_x <- ncol(X_bar_i)
  
  # Ensure X_bar_i is a matrix
  if (!is.matrix(X_bar_i)) {
    X_bar_i <- matrix(X_bar_i, ncol = 1)
  }
  
  # Center
  Z_mean <- colMeans(Z_panel)
  X_mean <- colMeans(X_bar_i)
  
  Zc <- sweep(Z_panel, 2, Z_mean)
  Xc <- sweep(X_bar_i, 2, X_mean)
  
  # Q_zz = (1/N) * Z_c' Z_c  (Eq. 8)
  Qzz <- crossprod(Zc) / N_g
  
  # V_zz = (1/N) * sum(chat_i^2 * (z_i - zbar)(z_i - zbar)')  (Eq. 19)
  Vzz <- matrix(0, k_z, k_z)
  for (i in seq_len(N_g)) {
    zi <- Zc[i, ]
    Vzz <- Vzz + chat[i]^2 * outer(zi, zi)
  }
  Vzz <- Vzz / N_g
  
  # Q_zxbar = (1/N) * Z_c' X_c  (Eq. 9)
  Qzxbar <- crossprod(Zc, Xc) / N_g
  
  # V_gamma = (1/N) * Qzz^{-1} * [Vzz + Qzxbar * N*V(beta) * Qzxbar'] * Qzz^{-1}
  Qzz_inv <- solve(Qzz)
  mid <- Vzz + Qzxbar %*% (N_g * V_beta_robust) %*% t(Qzxbar)
  V_gamma <- (1 / N_g) * Qzz_inv %*% mid %*% Qzz_inv
  
  # Clean up names
  rownames(V_gamma) <- colnames(Z_panel)
  colnames(V_gamma) <- colnames(Z_panel)
  
  V_gamma
}


#' FEF-IV Stage 2 estimation with instruments
#' @keywords internal
#' @noRd
.fef_iv_stage2 <- function(u_bar, Z, IV, panel_id, panels) {
  
  N_g <- length(panels)
  k_z <- ncol(Z)
  k_iv <- ncol(IV)
  
  if (k_iv < k_z) {
    stop("Number of instruments (", k_iv, ") must be >= number of ",
         "endogenous z-variables (", k_z, ")")
  }
  
  # Get one observation per panel (first row of each panel)
  first_row <- !duplicated(panel_id)
  Z_panel <- Z[first_row, , drop = FALSE]
  IV_panel <- IV[first_row, , drop = FALSE]
  
  # 2SLS: First stage - regress Z on IV
  IV_aug <- cbind(1, IV_panel)
  Z_hat <- matrix(0, N_g, k_z)
  for (j in seq_len(k_z)) {
    fit_j <- lm.fit(IV_aug, Z_panel[, j])
    Z_hat[, j] <- fitted(fit_j)
  }
  
  # Second stage - regress u_bar on Z_hat
  Z_hat_aug <- cbind(1, Z_hat)
  fit_iv <- lm.fit(Z_hat_aug, u_bar)
  
  coefs <- coef(fit_iv)
  alpha <- coefs[1]
  gamma <- coefs[-1]
  names(gamma) <- colnames(Z)
  
  # IV residuals: upsilon_i = ubar_i - alpha - Z_i' gamma_iv
  upsilon <- as.vector(u_bar) - alpha - as.vector(Z_panel %*% gamma)
  
  list(
    gamma = gamma,
    alpha = alpha,
    upsilon = as.vector(upsilon),
    Z_panel = Z_panel,
    IV_panel = IV_panel
  )
}


#' Pesaran-Zhou variance for gamma_IV (Eq. 51)
#' @keywords internal
#' @noRd
.pz_variance_fef_iv <- function(Z_panel, IV_panel, X_bar_i, upsilon,
                                V_beta_robust, N_g) {
  
  k_z <- ncol(Z_panel)
  k_iv <- ncol(IV_panel)
  k_x <- ncol(X_bar_i)
  
  if (!is.matrix(X_bar_i)) {
    X_bar_i <- matrix(X_bar_i, ncol = 1)
  }
  
  # Center all
  Z_mean <- colMeans(Z_panel)
  R_mean <- colMeans(IV_panel)
  X_mean <- colMeans(X_bar_i)
  
  Zc <- sweep(Z_panel, 2, Z_mean)
  Rc <- sweep(IV_panel, 2, R_mean)
  Xc <- sweep(X_bar_i, 2, X_mean)
  
  # Q_zr = (1/N) * Z_c' R_c
  Qzr <- crossprod(Zc, Rc) / N_g
  
  # Q_rr = (1/N) * R_c' R_c
  Qrr <- crossprod(Rc) / N_g
  
  # Q_rxbar = (1/N) * R_c' X_c
  Qrxbar <- crossprod(Rc, Xc) / N_g
  
  # V_rr = (1/N) * sum(upsilon_i^2 * (r_i - rbar)(r_i - rbar)')
  Vrr <- matrix(0, k_iv, k_iv)
  for (i in seq_len(N_g)) {
    ri <- Rc[i, ]
    Vrr <- Vrr + upsilon[i]^2 * outer(ri, ri)
  }
  Vrr <- Vrr / N_g
  
  # H_zr = (Q_zr * Q_rr^{-1} * Q_zr')^{-1} * Q_zr * Q_rr^{-1}
  Qrr_inv <- solve(Qrr)
  H_mid <- Qzr %*% Qrr_inv %*% t(Qzr)
  Hzr <- solve(H_mid) %*% Qzr %*% Qrr_inv
  
  # V_gamma = (1/N) * H_zr * [V_rr + Q_rxbar * N*V(beta) * Q_rxbar'] * H_zr'
  mid <- Vrr + Qrxbar %*% (N_g * V_beta_robust) %*% t(Qrxbar)
  V_gamma <- (1 / N_g) * Hzr %*% mid %*% t(Hzr)
  
  rownames(V_gamma) <- colnames(Z_panel)
  colnames(V_gamma) <- colnames(Z_panel)
  
  V_gamma
}


#' FEVD estimator
#' @keywords internal
#' @noRd
.estimate_fevd <- function(parsed) {
  
  # Stage 1: Fixed effects
  stage1 <- .fe_stage1(parsed$y, parsed$X, parsed$panel_id)
  
  # Stage 2: Decompose unit effects
  stage2 <- .fef_stage2(stage1$u_bar, parsed$Z, parsed$panel_id, parsed$panels)
  
  # Stage 3: Pooled OLS (point estimates equal FEF, SEs wrong - we don't use them)
  # We compute Pesaran-Zhou corrected SEs instead
  
  # Variance for gamma using Pesaran-Zhou Eq. 17
  V_gamma <- .pz_variance_fef(
    stage2$Z_panel, stage1$X_bar_i, stage2$chat,
    stage1$V_beta_robust, parsed$N_g
  )
  
  # Build combined coefficient vector
  k_x <- length(stage1$beta)
  k_z <- length(stage2$gamma)
  
  coefficients <- c(stage1$beta, stage2$gamma, "_cons" = stage2$alpha)
  
  # Build combined variance matrix (block diagonal)
  k_total <- k_x + k_z + 1
  V <- matrix(0, k_total, k_total)
  
  # Beta block
  V[1:k_x, 1:k_x] <- stage1$V_beta
  
  # Gamma block
  V[(k_x + 1):(k_x + k_z), (k_x + 1):(k_x + k_z)] <- V_gamma
  
  # Alpha variance (from Stage 2 OLS)
  Z_aug <- cbind(1, stage2$Z_panel)
  se_alpha <- sqrt(sum(stage2$chat^2) / (parsed$N_g - k_z - 1) *
                     solve(crossprod(Z_aug))[1, 1])
  V[k_total, k_total] <- se_alpha^2
  
  rownames(V) <- colnames(V) <- names(coefficients)
  
  # Fitted values
  fitted <- parsed$X %*% stage1$beta +
    parsed$Z %*% stage2$gamma +
    stage2$alpha
  
  list(
    coefficients = coefficients,
    vcov = V,
    beta = stage1$beta,
    gamma = stage2$gamma,
    intercept = stage2$alpha,
    residuals = stage1$e_it,
    fitted.values = as.vector(fitted),
    sigma2_e = stage1$sigma2_e,
    sigma2_u = stage1$sigma2_u,
    N = parsed$N,
    N_g = parsed$N_g,
    T_bar = parsed$T_bar,
    k_x = k_x,
    k_z = k_z,
    x_names = parsed$x_names,
    z_names = parsed$z_names,
    y_name = parsed$y_name,
    V_gamma_pz = V_gamma
  )
}


#' FEF estimator
#' @keywords internal
#' @noRd
.estimate_fef <- function(parsed) {
  
  # Stage 1: Fixed effects
  stage1 <- .fe_stage1(parsed$y, parsed$X, parsed$panel_id)
  
  # Stage 2: FEF regression
  stage2 <- .fef_stage2(stage1$u_bar, parsed$Z, parsed$panel_id, parsed$panels)
  
  # Pesaran-Zhou variance
  V_gamma <- .pz_variance_fef(
    stage2$Z_panel, stage1$X_bar_i, stage2$chat,
    stage1$V_beta_robust, parsed$N_g
  )
  
  # Build results (same structure as FEVD)
  k_x <- length(stage1$beta)
  k_z <- length(stage2$gamma)
  
  coefficients <- c(stage1$beta, stage2$gamma, "_cons" = stage2$alpha)
  
  k_total <- k_x + k_z + 1
  V <- matrix(0, k_total, k_total)
  V[1:k_x, 1:k_x] <- stage1$V_beta
  V[(k_x + 1):(k_x + k_z), (k_x + 1):(k_x + k_z)] <- V_gamma
  
  Z_aug <- cbind(1, stage2$Z_panel)
  se_alpha <- sqrt(sum(stage2$chat^2) / (parsed$N_g - k_z - 1) *
                     solve(crossprod(Z_aug))[1, 1])
  V[k_total, k_total] <- se_alpha^2
  
  rownames(V) <- colnames(V) <- names(coefficients)
  
  fitted <- parsed$X %*% stage1$beta +
    parsed$Z %*% stage2$gamma +
    stage2$alpha
  
  list(
    coefficients = coefficients,
    vcov = V,
    beta = stage1$beta,
    gamma = stage2$gamma,
    intercept = stage2$alpha,
    residuals = stage1$e_it,
    fitted.values = as.vector(fitted),
    sigma2_e = stage1$sigma2_e,
    sigma2_u = stage1$sigma2_u,
    N = parsed$N,
    N_g = parsed$N_g,
    T_bar = parsed$T_bar,
    k_x = k_x,
    k_z = k_z,
    x_names = parsed$x_names,
    z_names = parsed$z_names,
    y_name = parsed$y_name,
    V_gamma_pz = V_gamma
  )
}


#' FEF-IV estimator
#' @keywords internal
#' @noRd
.estimate_fef_iv <- function(parsed) {
  
  if (is.null(parsed$IV)) {
    stop("FEF-IV requires instruments. Provide 'instruments' argument.")
  }
  
  # Stage 1: Fixed effects
  stage1 <- .fe_stage1(parsed$y, parsed$X, parsed$panel_id)
  
  # Stage 2: FEF-IV regression
  stage2 <- .fef_iv_stage2(stage1$u_bar, parsed$Z, parsed$IV, parsed$panel_id, parsed$panels)
  
  # Pesaran-Zhou variance for IV (Eq. 51)
  V_gamma <- .pz_variance_fef_iv(
    stage2$Z_panel, stage2$IV_panel, stage1$X_bar_i, stage2$upsilon,
    stage1$V_beta_robust, parsed$N_g
  )
  
  k_x <- length(stage1$beta)
  k_z <- length(stage2$gamma)
  k_iv <- ncol(parsed$IV)
  
  coefficients <- c(stage1$beta, stage2$gamma, "_cons" = stage2$alpha)
  
  k_total <- k_x + k_z + 1
  V <- matrix(0, k_total, k_total)
  V[1:k_x, 1:k_x] <- stage1$V_beta
  V[(k_x + 1):(k_x + k_z), (k_x + 1):(k_x + k_z)] <- V_gamma
  
  # IV variance for alpha
  IV_aug <- cbind(1, stage2$IV_panel)
  se_alpha <- sqrt(sum(stage2$upsilon^2) / (parsed$N_g - k_iv - 1) *
                     MASS::ginv(crossprod(IV_aug))[1, 1])
  V[k_total, k_total] <- se_alpha^2
  
  rownames(V) <- colnames(V) <- names(coefficients)
  
  fitted <- parsed$X %*% stage1$beta +
    parsed$Z %*% stage2$gamma +
    stage2$alpha
  
  list(
    coefficients = coefficients,
    vcov = V,
    beta = stage1$beta,
    gamma = stage2$gamma,
    intercept = stage2$alpha,
    residuals = stage1$e_it,
    fitted.values = as.vector(fitted),
    sigma2_e = stage1$sigma2_e,
    sigma2_u = stage1$sigma2_u,
    N = parsed$N,
    N_g = parsed$N_g,
    T_bar = parsed$T_bar,
    k_x = k_x,
    k_z = k_z,
    k_iv = k_iv,
    x_names = parsed$x_names,
    z_names = parsed$z_names,
    iv_names = parsed$iv_names,
    y_name = parsed$y_name,
    V_gamma_pz = V_gamma
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{bw_ratio}
\alias{bw_ratio}
\title{Between/Within Variance Ratio for Time-Invariant Variables}
\usage{
bw_ratio(data, variables, id)
}
\arguments{
\item{data}{A data frame containing the panel data.}

\item{variables}{A character vector of variable names to analyze.}

\item{id}{Character string naming the panel identifier variable.}
}
\value{
A data frame with columns:
\describe{
\item{variable}{Variable name}
\item{sd_between}{Between-panel standard deviation}
\item{sd_within}{Within-panel standard deviation}
\item{bw_ratio}{Ratio of between to within SD}
}
}
\description{
Computes the between-panel and within-panel standard deviations for
specified variables, along with their ratio. This diagnostic helps assess
whether FEVD/FEF methods may improve upon standard FE estimation.
}
\details{
For truly time-invariant variables, the within-panel SD should be zero
(or near-zero due to numerical precision), giving an infinite ratio.

According to Plümper and Troeger (2007), FEVD/FEF methods tend to improve
upon standard FE when:
\itemize{
\item The between/within ratio exceeds approximately 1.7
\item The correlation between z and the unobserved unit effect is not too high
}
}
\examples{
# Create example data
set.seed(42)
N <- 50
T <- 5
id <- rep(1:N, each = T)
z_invariant <- rep(rnorm(N), each = T)  # Truly time-invariant
z_slow <- rep(rnorm(N), each = T) + rnorm(N * T, sd = 0.1)  # Slowly varying
x_varying <- rnorm(N * T)  # Time-varying

data <- data.frame(id = id, z_inv = z_invariant,
                   z_slow = z_slow, x = x_varying)

bw_ratio(data, c("z_inv", "z_slow", "x"), id = "id")

}
\references{
Plumper, T., & Troeger, V. E. (2007). Efficient Estimation of Time-Invariant
and Rarely Changing Variables in Finite Sample Panel Analyses with Unit Fixed
Effects. \emph{Political Analysis}, 15(2), 124-139.
\doi{10.1093/pan/mpm002}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xtfifevd-package.R
\docType{package}
\name{xtfifevd-package}
\alias{xtfifevd-package}
\title{xtfifevd: Panel Fixed Effects Filtered and Variance Decomposition Estimation}
\description{
Implements fixed effects estimators for time-invariant variables in panel
data models. Standard fixed effects (FE) estimation cannot identify
coefficients on time-invariant regressors because they are collinear with
the individual fixed effects. This package provides three methods to estimate
these coefficients:
}
\section{Methods}{

\describe{
\item{FEVD}{Fixed Effects Vector Decomposition (Plümper & Troeger, 2007).
A three-stage estimator that decomposes the unit effects into explained
(by time-invariant variables) and unexplained components.}
\item{FEF}{Fixed Effects Filtered (Pesaran & Zhou, 2016). A two-stage
estimator that regresses time-averaged FE residuals on time-invariant
variables.}
\item{FEF-IV}{Fixed Effects Filtered with Instrumental Variables
(Pesaran & Zhou, 2016). Uses external instruments when time-invariant
variables are endogenous.}
}
}

\section{Variance Estimation}{

All methods use the Pesaran-Zhou (2016) variance estimator (Equation 17 for
FEF/FEVD, Equation 51 for FEF-IV). These correct variance estimators account
for the generated regressor problem and avoid the severe size distortions
documented for naive pooled OLS standard errors.
}

\references{
Plumper, T., & Troeger, V. E. (2007). Efficient Estimation of Time-Invariant
and Rarely Changing Variables in Finite Sample Panel Analyses with Unit Fixed
Effects. \emph{Political Analysis}, 15(2), 124-139.
\doi{10.1093/pan/mpm002}

Pesaran, M. H., & Zhou, Q. (2018). Estimation of time-invariant effects in
static panel data models. \emph{Econometric Reviews}, 37(10), 1137-1171.
\doi{10.1080/07474938.2016.1222225}

Greene, W. H. (2011). Fixed Effects Vector Decomposition: A Magical Solution
to the Problem of Time-Invariant Variables in Fixed Effects Models?
\emph{Political Analysis}, 19(2), 135-146.
\doi{10.1093/pan/mpq034}
}
\author{
\strong{Maintainer}:  

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xtfifevd.R
\name{xtfifevd}
\alias{xtfifevd}
\alias{fevd}
\alias{fef}
\alias{fef_iv}
\title{Panel Fixed Effects Estimation for Time-Invariant Variables}
\usage{
xtfifevd(
  formula,
  data,
  id,
  time,
  method = c("fevd", "fef", "fef_iv"),
  instruments = NULL,
  na.action = na.omit
)

fevd(formula, data, id, time, na.action = na.omit)

fef(formula, data, id, time, na.action = na.omit)

fef_iv(formula, data, id, time, instruments, na.action = na.omit)
}
\arguments{
\item{formula}{A formula of the form \code{y ~ x1 + x2 | z1 + z2} where variables
before \code{|} are time-varying and variables after \code{|} are time-invariant.}

\item{data}{A data frame containing the variables.}

\item{id}{Character string naming the panel (individual) identifier variable.}

\item{time}{Character string naming the time identifier variable.}

\item{method}{Estimation method: \code{"fevd"} (default), \code{"fef"}, or \code{"fef_iv"}.}

\item{instruments}{For \code{method = "fef_iv"}, a one-sided formula specifying
instrumental variables, e.g., \code{~ iv1 + iv2}.}

\item{na.action}{How to handle missing values. Default is \code{na.omit}.}
}
\value{
An object of class \code{"xtfifevd"} containing:
\describe{
\item{coefficients}{Named vector of all coefficients (beta, gamma, intercept)}
\item{vcov}{Variance-covariance matrix using Pesaran-Zhou estimator}
\item{beta}{Coefficients on time-varying variables (from FE stage)}
\item{gamma}{Coefficients on time-invariant variables}
\item{intercept}{Overall intercept}
\item{residuals}{Idiosyncratic residuals from FE stage}
\item{fitted.values}{Fitted values}
\item{sigma2_e}{Variance of idiosyncratic error}
\item{sigma2_u}{Variance of unit effects}
\item{N}{Total number of observations}
\item{N_g}{Number of groups (panels)}
\item{T_bar}{Average time periods per panel}
\item{method}{Estimation method used}
\item{call}{The matched call}
}
}
\description{
Estimates panel models with time-invariant regressors using FEVD, FEF, or
FEF-IV methods. Standard fixed effects estimation cannot identify
coefficients on time-invariant variables; these methods decompose or filter
the unit effects to recover these coefficients.
}
\details{
\subsection{Model}{

The panel model is:
\deqn{y_{it} = \alpha_i + x_{it}'\beta + z_i'\gamma + \varepsilon_{it}}

where \eqn{x_{it}} are time-varying regressors, \eqn{z_i} are time-invariant
regressors, and \eqn{\alpha_i} are individual fixed effects.
}

\subsection{Stage 1 (All Methods)}{

Fixed effects regression of \eqn{y_{it}} on \eqn{x_{it}} yields consistent
\eqn{\hat{\beta}} and combined residuals \eqn{\hat{u}_{it} = \hat{\alpha}_i + \hat{\varepsilon}_{it}}.
}

\subsection{Stage 2}{

Time-averaged residuals \eqn{\bar{u}_i} are regressed on \eqn{z_i}:
\itemize{
\item \strong{FEF}: OLS of \eqn{\bar{u}_i} on \eqn{z_i} with intercept
\item \strong{FEF-IV}: 2SLS using instruments \eqn{r_i}
\item \strong{FEVD}: Same as FEF, then Stage 3 pooled OLS (point estimates identical)
}
}

\subsection{Variance Estimation}{

Uses Pesaran-Zhou (2016) Equation 17/51 which properly accounts for
estimation uncertainty from Stage 1. The naive pooled OLS standard errors
from FEVD Stage 3 are \strong{inconsistent} and can understate true SEs by
factors of 2-5x or more.
}
}
\section{Functions}{
\itemize{
\item \code{fevd}: FEVD estimation (3-stage, Plümper-Troeger)

\item \code{fef}: FEF estimation (2-stage, Pesaran-Zhou)

\item \code{fef_iv}: FEF-IV estimation with instruments

}}
\examples{
# Simulate panel data
set.seed(123)
N <- 100  # panels
T <- 10   # time periods
n <- N * T

# Generate data
id <- rep(1:N, each = T)
time <- rep(1:T, N)
alpha_i <- rep(rnorm(N), each = T)  # Fixed effects
z <- rep(rnorm(N), each = T)        # Time-invariant
x <- rnorm(n)                        # Time-varying
y <- 1 + 2 * x + 0.5 * z + alpha_i + rnorm(n, sd = 0.5)

data <- data.frame(id = id, time = time, y = y, x = x, z = z)

# Estimate with different methods
fit_fevd <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time")
summary(fit_fevd)

fit_fef <- xtfifevd(y ~ x | z, data = data, id = "id", time = "time",
                    method = "fef")
summary(fit_fef)

}
\references{
Plumper, T., & Troeger, V. E. (2007). Efficient Estimation of Time-Invariant
and Rarely Changing Variables in Finite Sample Panel Analyses with Unit Fixed
Effects. \emph{Political Analysis}, 15(2), 124-139.
\doi{10.1093/pan/mpm002}

Pesaran, M. H., & Zhou, Q. (2018). Estimation of time-invariant effects in
static panel data models. \emph{Econometric Reviews}, 37(10), 1137-1171.
\doi{10.1080/07474938.2016.1222225}
}
\seealso{
\code{\link[=fevd]{fevd}}, \code{\link[=fef]{fef}}, \code{\link[=fef_iv]{fef_iv}}, \code{\link[=bw_ratio]{bw_ratio}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{xtfifevd-methods}
\alias{xtfifevd-methods}
\title{Methods for xtfifevd objects}
\description{
Methods for xtfifevd objects
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimation.R
\name{estimation-internal}
\alias{estimation-internal}
\title{Internal estimation functions}
\description{
Internal estimation functions
}
\keyword{internal}
